; CircuitBread Tutorials
; Microcontroller Basics (PIC10F200), Part 11 - Communicating with a PC using UART
; Link: https://www.circuitbread.com/tutorials/communicating-with-a-pc-using-uart---part-11-microcontroller-basics-pic10f200
; Source code conversion from Microchip MPASM to Microchip XC8 PIC Assembler.

; Include target device specific definitions.
#include <xc.inc>

; Some equates not included in the device specific definitions.
;----- OPTION_REG Bits -----------------------------------------------------
T0CS        EQU 0005h
NOT_GPPU    EQU 0006h
;----- STATUS Bits -----------------------------------------------------
Z           EQU 0002h

; Set the configuration word.
CONFIG WDTE = OFF  ; Watchdog Timer (WDT disabled)
CONFIG CP = OFF    ; Code Protect (Code protection off)
CONFIG MCLRE = OFF ; Master Clear Enable (MCLR disabled, GP3 enabled)

i       EQU 10  ;Delay register 1
j       EQU 11  ;Delay register 2
rx_data EQU 12  ;Received byte
tx_data EQU 13  ;Byte to transmit
count   EQU 14  ;Bit counter for UART communication

; NOTE: To make sure the PIC10F200 RC oscillator calibration instruction at
; the 0xFF program memory address is not overwriten place this psect elsewhere
; , for example at the program memory start address: Project Properties >
; pic-as Global Options > Additional Options: -Wl,-pMyCode=0h
    PSECT MyCode,class=CODE,delta=2
INIT:
    MOVLW ~((1<<T0CS)|(1<<NOT_GPPU))
    OPTION              ;Enable GP2 and pull-ups
    MOVLW ~((1 << GPIO_GP0_POSITION) | (1 << GPIO_GP2_POSITION))    
    TRIS GPIO           ;Set GP0 and GP2 as outputs
    BSF GPIO, GPIO_GP2_POSITION     ;Set GP2 as output high
LOOP:
    BTFSC GPIO, GPIO_GP3_POSITION   ;Check if GP3 is 0 (SW1 is pressed)
    GOTO WAIT_RX        ;If it is not 0 then return
    CALL DEBOUNCE_DELAY ;Otherwise perform the debounce delay
    BTFSC GPIO, GPIO_GP3_POSITION   ;And check GP3 again
    GOTO WAIT_RX        ;If it is not 0 then return
    BTFSS GPIO, GPIO_GP3_POSITION   ;Otherwise wait while GP3 is 0
    GOTO $-1  	        ;Loop to wait
    CALL DEBOUNCE_DELAY ;After GP3 is set (SW1 is released)
SEND_MESSAGE:           ;Start of the sending message via UART
    MOVLW 'H'           ;Load the character to send
    CALL TX_BYTE        ;And call 'TX_BYTE' subroutine
    MOVLW 'e'           ;Quote marks means use ASCII equivalent
    CALL TX_BYTE
    MOVLW 'l'
    CALL TX_BYTE
    MOVLW 'l'
    CALL TX_BYTE
    MOVLW 'o'
    CALL TX_BYTE
    MOVLW 0Dh           ;'\r'
    CALL TX_BYTE
    MOVLW 0Ah           ;'\n'
    CALL TX_BYTE
WAIT_RX:
    BTFSS GPIO, GPIO_GP1_POSITION   ;Check the GP1 level, if 1 then skip next line
    CALL RX_BYTE        ;Otherwise receive the byte
    GOTO LOOP           ;Return to the 'LOOP' label
;--------------------------------------------------------------
TX_BYTE:
    MOVWF tx_data       ;into the 'tx_data' register
    CLRF count          ;Clear the 'count' register
    BCF GPIO, GPIO_GP2_POSITION     ;Beginning of the Start bit
    CALL UART_DELAY     ;One bit delay    
SHIFT_TX_DATA:          ;Start to shift the data byte
    BTFSC tx_data, 0    ;if the LSB of the 'tx_data' is 0
    GOTO SET_BIT        ;then skip this line,
    BCF GPIO, GPIO_GP2_POSITION     ;clear the GP2 bit
    GOTO DELAY          ;and go to the 'DELAY' label
SET_BIT:                ;If the LSB of the 'tx_data' is 1
    BSF GPIO, GPIO_GP2_POSITION     ;then set the GP2 bit
DELAY:
    CALL UART_DELAY     ;and perform one bit delay
    RRF tx_data, F      ;Shift the data byte to the one bit right
    INCF count, F       ;Increment the 'count' register
    BTFSS count, 3      ;Check if the third bit of the 'count' is 1 (count = 8)
    GOTO SHIFT_TX_DATA  ;If it is not set then return to the 'SHIFT_TX_DATA'
    BSF GPIO, GPIO_GP2_POSITION     ;Otherwise start the Stop bit
    CALL UART_DELAY     ;and perform one bit delay
    RETLW 0
;--------------------------------------------------------------
RX_BYTE:                ;Beginning of the RX_BYTE subroutine
    CALL HALF_UART_DELAY;Delay to the middle of the bit interval
    BTFSC GPIO, GPIO_GP1_POSITION   ;If the GP1 bit is not 0
    RETLW 0             ;then error of the Start bit, and return
    CLRF count          ;Else clear 'count' register
    CLRF rx_data        ;and clear 'rx_data' register
SHIFT_RX_DATA:          ;Start receiving the data byte
    CALL UART_DELAY     ;Call one bit delay
    RRF rx_data, F      ;Shift the 'rx_data' one bit to the right
    BTFSC GPIO,GPIO_GP1_POSITION    ;If GP1 bit is 1
    BSF rx_data, 7      ;then set the MSB of the 'rx_data' to 1
    INCF count, F       ;Increment the counter
    BTFSS count, 3      ;and check if it is 8
    GOTO SHIFT_RX_DATA  ;If it is not then return to the 'SHIFT_RX_DATA'
    CALL UART_DELAY     ;Otherwise call one bit delay
    BTFSS GPIO, GPIO_GP1_POSITION   ;And check the stop bit
    RETLW 0             ;if the GP1 is not 1 then return
    MOVLW '1'           ;Else load the '1' into the W register
    XORWF rx_data, W    ;And perform the XOR between W and 'rx_data'
    BTFSC STATUS, Z     ;If result is 0 (rx_data = W)
    BSF GPIO, GPIO_GP0_POSITION     ;then set the GP0 bit
    MOVLW '0'           ;Make the same check for the '0' value
    XORWF rx_data, W
    BTFSC STATUS, Z
    BCF GPIO, GPIO_GP0_POSITION     ;and clear the GP0 bit if rx_data = '0'
    RETLW 0
;--------------------------------------------------------------
UART_DELAY:             ;Start UART_DELAY subroutine here
    MOVLW 29            ;Load initial value for the delay
    MOVWF i             ;Copy the value to the register i
DELAY_LOOP_UART:        ;Start delay loop
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP_UART;If not, go to the DELAY_LOOP_UART label
    RETLW 0             ;Else return from the subroutine

HALF_UART_DELAY:        ;Start HALF_UART_DELAY subroutine here

    MOVLW 16            ;Load initial value for the delay
    MOVWF i             ;Copy the value to the register i
DELAY_LOOP_HALF:        ;Start delay loop
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP_HALF;If not, go to the DELAY_LOOP_HALF label
    RETLW 0             ;Else return from the subroutine

DEBOUNCE_DELAY:         ;Start DEBOUNCE_DELAY subroutine here
    MOVLW 40            ;Load initial value for the delay
    MOVWF i             ;Copy the value to the register i
    MOVWF j             ;Copy the value to the register j
DELAY_LOOP:             ;Start delay loop
    DECFSZ i, F         ;Decrement i and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    DECFSZ j, F         ;Decrement j and check if it is not zero
    GOTO DELAY_LOOP     ;If not, then go to the DELAY_LOOP label
    RETLW 0             ;Else return from the subroutine

    END INIT        ; Program entry point.